(**************************************************************************)
(*                                                                        *)
(*      whitelister : a Whitelister Policy Daemon for Postfix             *)
(*      ~~~~~~~~~~~                                                       *)
(*                                                                        *)
(*  Copyright (C) 2005 Polytechnique.org                                  *)
(*  Author : Pierre Habouzit                                              *)
(*                                                                        *)
(*  This program is free software; you can redistribute it and/or modify  *)
(*  it under the terms of the GNU General Public License as published by  *)
(*  the Free Software Foundation; either version 2 of the License, or     *)
(*  (at your option) any later version.                                   *)
(*                                                                        *)
(*  This program is distributed in the hope that it will be useful,       *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU General Public License for more details.                          *)
(*                                                                        *)
(*  You should have received a copy of the GNU General Public License     *)
(*  along with this program; if not, write to the Free Software           *)
(*  Foundation, Inc.,                                                     *)
(*  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA               *)
(*                                                                        *)
(**************************************************************************)

exception ParseError
exception Unknown

type t = (string, string) Hashtbl.t

(* Private *)
           
let get h k =
  try Hashtbl.find h k
  with Not_found -> raise Unknown
              
let check_policy pcy =
  try
    if get pcy "request" = "smtpd_access_policy" then
      pcy
    else
      raise ParseError
  with Not_found -> raise ParseError
           
(* public *)

let read ic =
  let res = Hashtbl.create 13 in
  try
    while true do
      try
        let line = input_line ic in
        if String.length line = 0 then raise End_of_file;
        let i = String.index line '=' in
        Hashtbl.add res (Str.string_before line i) (Str.string_after line (i+1))
      with Not_found -> raise ParseError
    done;
    assert false
  with
    | End_of_file -> check_policy res
    | except -> raise except

let clear = Hashtbl.clear

let client_address h = get h "client_address"
let client_name    h = get h "client_name"
let sender         h = get h "sender"
let helo_name      h = get h "helo_name"

